//
//  mail.c
//  FemMonitor
//
//  Created by James Shiffer on 3/22/21.
//  Copyright © 2021 FemboyFinancial. All rights reserved.
//

#include <curl/curl.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "common.h"
#include "mail.h"

static CURL* curl = NULL;
static WINDOW* window = NULL;
static int width;
static int height;


typedef struct _mail_info {
    char uptime[64];
    unsigned users;
    char disk_used[8];
    char disk_total[8];
} mail_info_t;

void mail_init(WINDOW* win, int h, int w)
{
    curl = curl_easy_init();
    window = win;
    width = w;
    height = h;
    
    mvwprintw(window, 0, START_COL, " E-Mail ");
    mvwprintw(window, START_ROW, START_COL, "Pinging...");
    if (curl == NULL)
    {
        wattron(window, COLOR_PAIR(COLORS_FAILURE));
        mvwprintw(window, 0, TITLE_START_COL, " ERROR ");
        mvwprintw(window, START_ROW, START_COL, "Failed to initialize libcurl!");
        wattroff(window, COLOR_PAIR(COLORS_FAILURE));
        curl_global_cleanup();
    }
    
    wrefresh(window);
}

bool parse_mail(const char* json, mail_info_t* time)
{
    const char* prop = "\"uptime\":\"";
    const char* read = strstr(json, prop);
    if (read == NULL)
        return FALSE;
    read += strlen(prop);
    const char* endprop = strstr(read, "\"");
    if (endprop == NULL)
        return FALSE;
    strncpy(time->uptime, read, endprop - read);

    prop = "\"users\":";
    read = strstr(json, prop);
    if (read == NULL)
        return FALSE;
    read += strlen(prop);
    endprop = strstr(read, ",");
    if (endprop == NULL)
        return FALSE;
    char users_str[8] = {0};
    strncpy(users_str, read, endprop - read);
    time->users = atoi(users_str);

    prop = "\"disk_used\":\"";
    read = strstr(json, prop);
    if (read == NULL)
        return FALSE;
    read += strlen(prop);
    endprop = strstr(read, "\"");
    if (endprop == NULL)
        return FALSE;
    strncpy(time->disk_used, read, endprop - read);

    prop = "\"disk_total\":\"";
    read = strstr(json, prop);
    if (read == NULL)
        return FALSE;
    read += strlen(prop);
    endprop = strstr(read, "\"");
    if (endprop == NULL)
        return FALSE;
    strncpy(time->disk_total, read, endprop - read);

    return TRUE;
}


void* mail_refresh(void* arg)
{
        // get mail server status
	CURLcode res;
        struct MemoryStruct mail_data = geturl(curl, "https://mail.femboyfinancial.jp/status.php", &res);

	if (res != CURLE_OK)
	{
		on_curl_error(window, res);
		goto cleanup;
	}
	
	// clear status lines
	for (int i = TITLE_START_COL - 1; i < TITLE_START_COL + 7; ++i)
		mvwprintw(window, 0, i, " ");
	
	for (int i = START_ROW; i < START_ROW+3; ++i)
	{
		for (int j = START_COL; j < width - 1; ++j)
			mvwprintw(window, i, j, " ");
	}
        
	mail_info_t mail = {0};
        if (parse_mail(mail_data.memory, &mail))
        {
            wattron(window, COLOR_PAIR(COLORS_SUCCESS));
            mvwprintw(window, 0, TITLE_START_COL, " ONLINE ");
            wattroff(window, COLOR_PAIR(COLORS_SUCCESS));

            mvwprintw(window, START_ROW, START_COL, "Uptime: %s", mail.uptime);
            mvwprintw(window, START_ROW+1, START_COL, "Users: %d", mail.users);
            mvwprintw(window, START_ROW+2, START_COL, "Disk: %s / %s used", mail.disk_used, mail.disk_total);
        }
        else
        {
            wattron(window, COLOR_PAIR(COLORS_FAILURE));
	    mvwprintw(window, START_ROW, START_COL, "Couldn't retrieve mail server status");
            wattroff(window, COLOR_PAIR(COLORS_FAILURE));
        }
        
        // refresh view
        wrefresh(window);

    cleanup:
        free(mail_data.memory);
}

void mail_destroy(void)
{
    if (curl != NULL)
        curl_easy_cleanup(curl);
}
